@@ -1,7 +1,7 @@
 __author__ = 'Altertech, http://www.altertech.com/'
 __copyright__ = 'Copyright (C) 2018-2020 Altertech Group'
 __license__ = 'Apache License 2.0'
-__version__ = '2.0.40'
+__version__ = '2.0.41'
 
 import platform
 import os
@@ -941,5 +941,6 @@ def bucket_cleanup():
         d = time.time()
     else:
         d = datetime.datetime.now()
-        q = 'bucket.cleanup:sqlite' if _d.db.name == 'sqlite' else 'bucket.cleanup'
+        q = 'bucket.cleanup:sqlite' if _d.db.name == 'sqlite' else \
+                'bucket.cleanup'
     _d.db.query(q, d=d)
